﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using HeuristicLab.Services.Hive.Common.DataTransfer;
using HeuristicLab.Services.Hive.Common.ServiceContracts;
using HeuristicLab.Clients.Hive.Slave.Tests;
using HeuristicLab.Clients.Hive;
using HeuristicLab.Services.Hive.DataAccess;

namespace HeuristicLab.Services.Hive.Tests {
  using DA = HeuristicLab.Services.Hive.DataAccess;
  using DT = HeuristicLab.Services.Hive.Common.DataTransfer;

  [TestClass]
  public class DaoTests {

    [ClassInitialize]
    public static void MyClassInitialize(TestContext testContext) {
      PluginLoader.pluginAssemblies.Any();
      ServiceLocator.Instance = new MockServiceLocator(ServiceLocator.Instance);
    }
    
    private IHiveService GetLocalService() {
      return new HiveService();
    }

    [TestMethod]
    public void TestJobDao() {
      IHiveDao dao = ServiceLocator.Instance.HiveDao;
      
      DT.Job job1 = new DT.Job();
      job1.DateCreated = DateTime.Now;
      job1.Id = dao.AddJob(job1);

      DT.Job job1loaded = dao.GetJob(job1.Id);
      Assert.AreEqual(job1.Id, job1loaded.Id);
      Assert.AreEqual(job1.CoresNeeded, job1loaded.CoresNeeded);
      Assert.AreEqual(null, job1loaded.DateCalculated);
      Assert.AreEqual(job1.DateCreated.ToString(), job1loaded.DateCreated.ToString());
      Assert.AreEqual(null, job1loaded.DateFinished);
      
      dao.DeleteJob(job1.Id);

      Assert.AreEqual(null, dao.GetJob(job1.Id));
    }

    [TestMethod]
    public void TestSlaveDao() {
      IHiveDao dao = ServiceLocator.Instance.HiveDao;
      DT.SlaveGroup slaveGroup = new DT.SlaveGroup() {
        Name = "Test"
      };
      slaveGroup.Id = dao.AddSlaveGroup(slaveGroup);

      DT.Slave slave = new DT.Slave() {
        Name = "Test"
      };
      slave.Id= dao.AddSlave(slave);

      dao.DeleteSlave(slave.Id);
      dao.DeleteSlaveGroup(slaveGroup.Id);
    }

    [TestMethod]
    public void TestJobDaoWaiting() {
      IHiveDao dao = ServiceLocator.Instance.HiveDao;

      DT.Job job = new DT.Job();
      job.DateCreated = DateTime.Now;
      job.JobState = JobState.Waiting;
      job.Id = dao.AddJob(job);


    }
  }
}
